@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py,v 1.45 2006/06/10 00:54:47 wwalker Exp $"
-__version__   = "$Revision: 1.45 $"
-__date__      = "$Date: 2006/06/10 00:54:47 $"
+__id__        = "$Id: braillegenerator.py,v 1.49 2006/06/21 15:05:42 richb Exp $"
+__version__   = "$Revision: 1.49 $"
+__date__      = "$Date: 2006/06/21 15:05:42 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1026,19 +1026,37 @@ class BrailleGenerator:
                 parent = obj.parent
                 row = parent.table.getRowAtIndex(obj.index)
                 column = parent.table.getColumnAtIndex(obj.index)
-                focusRowRegion = None
-                for i in range(0, parent.table.nColumns):
-                    accRow = parent.table.getAccessibleAt(row, i)
-                    cell = atspi.Accessible.makeAccessible(accRow)
-                    [cellRegions, focusRegion] = \
-                        self._getDefaultBrailleRegions(util.getRealActiveDescendant(cell))
-                    if len(rowRegions):
-                        rowRegions.append(braille.Region(" "))
-                    rowRegions.append(cellRegions[0])
-                    if i == column:
-                        focusRowRegion = cellRegions[0]
-                regions = [rowRegions, focusRowRegion]
-                settings.brailleVerbosityLevel = savedBrailleVerbosityLevel
+
+                # This is an indication of whether we should speak all the
+                # table cells (the user has moved focus up or down a row),
+                # or just the current one (focus has moved left or right in
+                # the same row).
+                #
+                speakAll = True
+                if parent.__dict__.has_key("lastRow") and \
+                    parent.__dict__.has_key("lastColumn"):
+                    speakAll = (parent.lastRow != row) or \
+                           ((row == 0 or row == parent.table.nRows-1) and \
+                            parent.lastColumn == column)
+
+                if speakAll == True:
+                    focusRowRegion = None
+                    for i in range(0, parent.table.nColumns):
+                        accRow = parent.table.getAccessibleAt(row, i)
+                        cell = atspi.Accessible.makeAccessible(accRow)
+                        [cellRegions, focusRegion] = \
+                            self._getDefaultBrailleRegions(\
+                                util.getRealActiveDescendant(cell))
+                        if len(rowRegions):
+                            rowRegions.append(braille.Region(" "))
+                        rowRegions.append(cellRegions[0])
+                        if i == column:
+                            focusRowRegion = cellRegions[0]
+                    regions = [rowRegions, focusRowRegion]
+                    settings.brailleVerbosityLevel = savedBrailleVerbosityLevel
+                else:
+                    regions = self._getDefaultBrailleRegions(
+                        util.getRealActiveDescendant(obj))
             else:
                 regions = self._getDefaultBrailleRegions(
                     util.getRealActiveDescendant(obj))
