@@ -19,22 +19,26 @@
 
 """Provides a SpeechServer factory for gnome-speech drivers."""
 
-__id__        = "$Id: gnomespeechfactory.py,v 1.34 2006/06/10 00:54:47 wwalker Exp $"
-__version__   = "$Revision: 1.34 $"
-__date__      = "$Date: 2006/06/10 00:54:47 $"
+__id__        = "$Id: gnomespeechfactory.py,v 1.44 2006/07/09 18:13:28 wwalker Exp $"
+__version__   = "$Revision: 1.44 $"
+__date__      = "$Date: 2006/07/09 18:13:28 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import string
+
 import bonobo
 
 import atspi
 import debug
+import punctuation_settings
 import settings
 import speech
 import speechserver
 
 from acss import ACSS
 from chnames import chnames
+from punctuation_settings import punctuation
 
 from orca_i18n import _           # for gettext support
 
@@ -365,6 +369,16 @@ class SpeechServer(speechserver.SpeechServer):
 
         s = self.__driver.createSpeaker(voice)
         speaker = _Speaker(s._narrow(GNOME.Speech.Speaker))
+
+        # Turn off punctuation if the speaker allows us to do so.  We
+        # do this because we want to handle spoken punctuation on our
+        # own.
+        #
+        try:
+            speaker.setParameterValue("punctuation mode", 0.0)
+        except:
+            pass
+
         speaker.registerCallback(self)
 
         parameters = speaker.getSupportedParameters()
@@ -527,6 +541,66 @@ class SpeechServer(speechserver.SpeechServer):
                 self.speak(text, acss, interrupt and (i == 0))
             i += 1
 
+    def __addVerbalizedPunctuation(self, oldText):
+        """Depending upon the users verbalized punctuation setting,
+        adjust punctuation symbols in the given text to their pronounced
+        equivalents. The pronounced text will either replace the
+        punctuation symbol or be inserted before it. In the latter case,
+        this is to retain spoken prosity.
+
+        If we are moving around by single characters, then always speak
+        the punctuation. We try to detect this by looking for just a
+        single character being spoken.
+
+        Arguments:
+        - oldText:      text to be parsed for punctuation.
+
+        Returns a text string with the punctuation symbols adjusted accordingly.
+        """
+
+        ## Replace ellipses (both manual and unicode) with "dot dot dot"
+        ##
+        oldText = oldText.replace("...", _(" dot dot dot"), 1)
+        oldText = oldText.replace("\342\200\246",  _(" dot dot dot"), 1)
+
+        style = settings.verbalizePunctuationStyle
+        newText = ''
+        for i in range(0, len(oldText)):
+            try:
+                level, action = punctuation[oldText[i]]
+
+                # Special case for punctuation in text like filenames or URL's:
+                #
+                isPrev = isNext = isSpecial = False
+                if i > 0:
+                    isPrev = not (oldText[i - 1] in string.whitespace)
+                if i < (len(oldText) - 1):
+                    isNext = not (oldText[i + 1] in string.whitespace)
+
+                # If this is a period and there is a non-space character
+                # on either side of it, then always speak it.
+                #
+                isSpecial = isPrev and isNext and (oldText[i] == ".")
+
+                if (len(oldText) == 1) or isSpecial or (style <= level):
+                    if isPrev:
+                        newText += " "
+                    newText += chnames[oldText[i]]
+                    if (action == punctuation_settings.PUNCTUATION_INSERT) \
+                        and not isNext:
+                        newText += oldText[i]
+                    if isNext:
+                        newText += " "
+                else:
+                    newText += oldText[i]
+            except:
+                if (len(oldText) == 1) and chnames.has_key(oldText[i]):
+                    newText += chnames[oldText[i].lower()]
+                else:
+                    newText += oldText[i]
+
+        return newText
+
     def __speak(self, text=None, acss=None, interrupt=True):
         """Speaks all queued text immediately.  If text is not None,
         it is added to the queue before speaking.
@@ -557,28 +631,7 @@ class SpeechServer(speechserver.SpeechServer):
                 speech.stop()
             return -1
 
-        # If the text to speak is a single character, see if we have a
-        # customized character pronunciation
-        #
-        if len(text) == 1:
-            try:
-                text = chnames[text.lower()]
-            except:
-                pass
-        else:
-            # Replace ellipses (both manual and unicode) with "dot dot dot"
-            #
-            text = text.replace("...", _(" dot dot dot"), 1)
-            text = text.replace("\342\200\246",  _(" dot dot dot"), 1)
-
-        # [[[TODO: HACK - We're going to add whitespace after "["
-        # because DECtalk wants to interpret these as inline DECtalk
-        # commands.]]]
-        #
-        # [[[UPDATE: commented this out because gnome-speech-0.4.0 now
-        # tells DECtalk to do a [:phon off], reducing this problem.]]]
-        #
-        #text = text.replace("[", "[ ")
+        text = self.__addVerbalizedPunctuation(text)
 
         try:
             # [[[TODO: WDW - back this stop out for now.  The problem is
