@@ -19,9 +19,9 @@
 
 """Custom script for StarOffice and OpenOffice."""
 
-__id__        = "$Id: StarOffice.py,v 1.34 2006/06/10 00:54:48 wwalker Exp $"
-__version__   = "$Revision: 1.34 $"
-__date__      = "$Date: 2006/06/10 00:54:48 $"
+__id__        = "$Id: StarOffice.py,v 1.42 2006/06/21 19:36:53 richb Exp $"
+__version__   = "$Revision: 1.42 $"
+__date__      = "$Date: 2006/06/21 19:36:53 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -157,13 +157,12 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
         Returns a list of utterances to be spoken for the object.
         """
 
-        speechGen = speechgenerator.SpeechGenerator
-        utterances = speechGen._getSpeechForTableCell(self, obj, 
-                                                      already_focused)
-
         global inputLineForCell, isSpreadSheetCell, locateInputLine
 
         if isSpreadSheetCell(obj):
+            utterances = []
+            utterances.append(util.getDisplayedText(\
+                    util.getRealActiveDescendant(obj)))
 
             if inputLineForCell == None:
                 inputLineForCell = locateInputLine(obj)
@@ -177,6 +176,10 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
                 objectText = obj.text.getText(0, -1)
                 if objectText and len(objectText) != 0:
                     utterances.append(" " + obj.name)
+        else:
+            speechGen = speechgenerator.SpeechGenerator
+            utterances = speechGen._getSpeechForTableCell(self, obj,
+                                                      already_focused)
 
         return utterances
 
@@ -404,9 +407,58 @@ class Script(default.Script):
         if self.endOfLink(obj, word, startOffset, endOffset):
             speech.speak(_("link"))
 
+    def isSetupDialog(self, obj):
+        """ Check to see if this object is in the Setup dialog by walking 
+        back up the object hierarchy until we get to the dialog object and 
+        checking to see if it has a name that starts with "Welcome to 
+        StarOffice".
+
+        Arguments:
+        - obj: an Accessible object that implements the AccessibleText
+               interface
+
+        Returns an indication of whether this object is in the Setup dialog.
+        """
+
+        found = False
+        while obj and obj.role != rolenames.ROLE_APPLICATION:
+            if obj.role == rolenames.ROLE_DIALOG and \
+                (obj.name and obj.name.startswith(_("Welcome to StarOffice"))):
+                debug.println(self.debugLevel, 
+                              "StarOffice.isSetupDialog: True.")
+                found = True
+
+            obj = obj.parent
+
+        return found
+
+    def speakSetupLabel(self, label):
+        """Speak this Setup dialog label.
+
+        Arguments:
+        - label: the Setup dialog Label.
+        """
+
+        text = util.getDisplayedText(label)
+        if text:
+            speech.speak(text)
+
+    def handleSetupPanel(self, panel):
+        """Find all the labels in this Setup panel and speak them.
+
+        Arguments:
+        - panel: the Setup panel.
+        """
+
+        allLabels = util.findByRole(panel, rolenames.ROLE_LABEL)
+        for i in range(0, len(allLabels)):
+            self.speakSetupLabel(allLabels[i])
+
     # This method tries to detect and handle the following cases:
     # 1) Writer: text paragraph.
     # 2) Writer: spell checking dialog.
+    # 3) Welcome to StarOffice dialog.
+    # 4) Calc - spread sheet table.
 
     def locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus):
         """Called when the visual object with focus changes.
@@ -451,6 +503,8 @@ class Script(default.Script):
             #
             hypertext = event.source.hypertext
             if not hypertext or (hypertext.getNLinks() == 0):
+                if settings.enableSpeechIndentation:
+                    self.speakTextIndentation(event.source, result[0])
                 speech.speak(result[0], None, False)
             else:
                 started = False
@@ -499,17 +553,215 @@ class Script(default.Script):
             pane = event.source.parent
             if pane.name.startswith(_("Spellcheck:")):
                 debug.println(self.debugLevel,
-                    "StarOffice.locusOfFocusChanged - Writer: spell check dialog.")
+                    "StarOffice.locusOfFocusChanged - " \
+                    + "Writer: spell check dialog.")
 
                 self.readMisspeltWord(event, pane)
 
                 # Fall-thru to process the event with the default handler.
 
+        # 3) Welcome to StarOffice dialog.
+        #
+        # Check to see if the object that just got focus is in the Setup
+        # dialog. If it is, then check for a variety of scenerios.
+
+        if self.isSetupDialog(event.source):
+
+            # Check for 2. License Agreement: Scroll Down button.
+            #
+            rolesList = [rolenames.ROLE_PUSH_BUTTON, \
+                         rolenames.ROLE_PANEL, \
+                         rolenames.ROLE_OPTION_PANE, \
+                         rolenames.ROLE_DIALOG, \
+                         rolenames.ROLE_APPLICATION]
+            if util.isDesiredFocusedItem(event.source, rolesList):
+                debug.println(self.debugLevel,
+                    "StarOffice.locusOfFocusChanged - Setup dialog: " \
+                    + "License Agreement screen: Scroll Down button.")
+                self.handleSetupPanel(event.source.parent)
+                speech.speak(_("Note that the Scroll Down button has to be pressed numerous times."))
+
+            # Check for 2. License Agreement: Accept button.
+            #
+            rolesList = [rolenames.ROLE_UNKNOWN, \
+                         rolenames.ROLE_SCROLL_PANE, \
+                         rolenames.ROLE_PANEL, \
+                         rolenames.ROLE_OPTION_PANE, \
+                         rolenames.ROLE_DIALOG, \
+                         rolenames.ROLE_APPLICATION]
+            if util.isDesiredFocusedItem(event.source, rolesList):
+                debug.println(self.debugLevel,
+                    "StarOffice.locusOfFocusChanged - Setup dialog: " \
+                    + "License Agreement screen: accept button.")
+                speech.speak(_("License Agreement Accept button now has focus."))
+
+            # Check for 3. Personal Data: Transfer Personal Data check box.
+            #
+            rolesList = [rolenames.ROLE_CHECK_BOX, \
+                         rolenames.ROLE_PANEL, \
+                         rolenames.ROLE_OPTION_PANE, \
+                         rolenames.ROLE_DIALOG, \
+                         rolenames.ROLE_APPLICATION]
+            if util.isDesiredFocusedItem(event.source, rolesList):
+                debug.println(self.debugLevel,
+                    "StarOffice.locusOfFocusChanged - Setup dialog: " \
+                    + "Personal Data: Transfer Personal Data check box.")
+                self.handleSetupPanel(event.source.parent)
+
+            # Check for 4. User name: First Name text field.
+            #
+            rolesList = [rolenames.ROLE_TEXT, \
+                        rolenames.ROLE_PANEL, \
+                        rolenames.ROLE_OPTION_PANE, \
+                        rolenames.ROLE_DIALOG, \
+                        rolenames.ROLE_APPLICATION]
+            if util.isDesiredFocusedItem(event.source, rolesList) and \
+               event.source.name == _("First name"):
+                debug.println(self.debugLevel,
+                    "StarOffice.locusOfFocusChanged - Setup dialog: " \
+                    + "User name: First Name text field.")
+
+                # Just speak the informative labels at the top of the panel
+                # (and not the ones that have LABEL_FOR relationships).
+                #
+                panel = event.source.parent
+                allLabels = util.findByRole(panel, rolenames.ROLE_LABEL)
+                for i in range(0, len(allLabels)):
+                    relations = allLabels[i].relations
+                    hasLabelFor = False
+                    for relation in relations:
+                        if relation.getRelationType() \
+                               == atspi.Accessibility.RELATION_LABEL_FOR:
+                            hasLabelFor = True
+                    if not hasLabelFor:
+                        self.speakSetupLabel(allLabels[i])
+
+            # Check for 5. Registration: Register Now radio button.
+            #
+            rolesList = [rolenames.ROLE_RADIO_BUTTON, \
+                        rolenames.ROLE_PANEL, \
+                        rolenames.ROLE_OPTION_PANE, \
+                        rolenames.ROLE_DIALOG, \
+                        rolenames.ROLE_APPLICATION]
+            if util.isDesiredFocusedItem(event.source, rolesList):
+                debug.println(self.debugLevel,
+                    "StarOffice.locusOfFocusChanged - Setup dialog: " \
+                    + "Registration: Register Now radio button.")
+                self.handleSetupPanel(event.source.parent)
+
+        # 4) Calc - spread sheet table.
+        #
+        # If we are in the Calc spread sheet table, and we want to read all
+        # of the row (as opposed to a single cell), then generate the braille 
+        # and speech for each of the showing spread sheet cells in the current
+        # row.
+        #
+        rolesList = [rolenames.ROLE_TABLE, \
+                     rolenames.ROLE_UNKNOWN, \
+                     rolenames.ROLE_SCROLL_PANE, \
+                     rolenames.ROLE_PANEL, \
+                     rolenames.ROLE_ROOT_PANE, \
+                     rolenames.ROLE_FRAME, \
+                     rolenames.ROLE_APPLICATION]
+        if settings.readTableCellRow and \
+           util.isDesiredFocusedItem(event.source, rolesList):
+            debug.println(self.debugLevel, "StarOffice.locusOfFocusChanged - " \
+                          + "Calc: spread sheet table - read row.")
+            table = event.source.table
+            row = table.getRowAtIndex(newLocusOfFocus.index)
+            column = table.getColumnAtIndex(newLocusOfFocus.index)
+
+            # This is an indication of whether we should speak all the
+            # table cells (the user has moved focus up or down a row),
+            # or just the current one (focus has moved left or right in
+            # the same row).
+            #
+            speakAll = True
+            if event.source.__dict__.has_key("lastRow") and \
+                event.source.__dict__.has_key("lastColumn"):
+                speakAll = (event.source.lastRow != row) or \
+                       ((row == 0 or row == event.source.table.nRows-1) and \
+                        event.source.lastColumn == column)
+
+            if speakAll == True:
+                utterances = []
+                regions = []
+
+                # [[[TODO: richb - one speedup improvement we can do here is 
+                # to look left and look right along the row from the current 
+                # column, to just get the range of cells to show.]]]
+                #
+                for i in range(0, table.nColumns):
+                    obj = table.getAccessibleAt(row, i)
+                    cell = atspi.Accessible.makeAccessible(obj)
+
+                    showing = cell.state.count(atspi.Accessibility.STATE_SHOWING)
+                    while cell.childCount:
+                        cell = cell.child(0)
+
+                    if showing:
+                        [cellRegions, focusedRegion] = \
+                            brailleGen.getBrailleRegions(cell)
+                        regions.extend(cellRegions)
+                        regions.append(braille.Region(" "))
+                        if column == i:
+                            cellWithFocus = focusedRegion
+
+                        result = speechGen.getSpeech(cell, True)
+                        utterances.append(result[0])
+
+                braille.displayRegions([regions, cellWithFocus])
+                speech.speakUtterances(utterances)
+                event.source.lastColumn = column
+                event.source.lastRow = row
+                return
+
         # Pass the event onto the parent class to be handled in the default way.
 
         default.Script.locusOfFocusChanged(self, event,
                                            oldLocusOfFocus, newLocusOfFocus)
 
+    # This method tries to detect and handle the following cases:
+    # 1) Setup dialog.
+
+    def onWindowActivated(self, event):
+        """Called whenever a property on an object changes.
+
+        Arguments:
+        - event: the Event
+        """
+
+        debug.printObjectEvent(self.debugLevel,
+                               event,
+                               event.source.toString())
+
+        # util.printAncestry(event.source)
+
+        # 1) Setup dialog.
+        #
+        # Check to see if the Setup dialog window has just been activated.
+        # If it has, then find the panel within it that has no name and
+        # speak all the labels within that panel.
+        #
+        if self.isSetupDialog(event.source):
+            debug.println(self.debugLevel,
+                "StarOffice.onWindowActivated - Setup dialog: Welcome screen.")
+
+            allPanels = util.findByRole(event.source.parent,
+                                        rolenames.ROLE_PANEL)
+            for i in range(0, len(allPanels)):
+                if not allPanels[i].name:
+                    allLabels = util.findByRole(allPanels[i], 
+                                                rolenames.ROLE_LABEL)
+                    for i in range(0, len(allLabels)):
+                        self.speakSetupLabel(allLabels[i])
+        else:
+            # Pass the event onto the parent class to be handled in the 
+            # default way.
+            #
+            default.Script.onWindowActivated(self, event)
+
+
     # This method tries to detect and handle the following cases:
     # 1) Writer: spell checking dialog.
 
@@ -586,6 +838,7 @@ class Script(default.Script):
         #
         rolesList = [rolenames.ROLE_LIST, \
                      rolenames.ROLE_COMBO_BOX, \
+                     rolenames.ROLE_PANEL, \
                      rolenames.ROLE_TOOL_BAR, \
                      rolenames.ROLE_PANEL, \
                      rolenames.ROLE_ROOT_PANE, \
@@ -608,8 +861,9 @@ class Script(default.Script):
                         if cellText and len(cellText) != 0:
                             if inputLineForCell and inputLineForCell.text:
                                 inputLine = inputLineForCell.text.getText(0,-1)
-                                if inputLine and len(inputLine) != 0:
-                                    if inputLine != cellText:
+                                if inputLine and \
+                                    (len(inputLine) > 1) and \
+                                    (inputLine[0] == "="):
                                         hf = _(" has formula")
                                         speech.speak(hf, None, False)
 
